class LifeGrid {
  PImage dog, cat, fish, bird;//cell images
  int gridWidth, gridHeight;//store grid info
  
  int communeThreshold = 2;//game of life rules
  int starveThreshold = 3;
  int birthVal = 3;
  
  LifeSquare[][] lifeField,lifeField2;//array of grid squares
  
  LifeGrid(int w,int h) {//constructor
    
    gridWidth = w;//init size
    gridHeight = h;
    
    lifeField = new LifeSquare[gridWidth][gridHeight];//create squares
    
    for (int x = 0; x < gridWidth; x++){
      for (int y = 0; y < gridHeight; y++){
        lifeField[x][y] = new LifeSquare(false);//init square as not alive
      }
    }
  }
  
  LifeSquare getSquare(int x, int y){//get square x,y
    return lifeField[x][y];
  }
  
  void displayState(int cellsize, color c, color c2){//display grid
    //init colour to default
    int fillColour = 0;
    for (int x = 0; x < gridWidth; x++){
      for (int y = 0; y < gridHeight; y++){
         if(lifeField[x][y].isItAlive()){//if cell is active
           print("1");//debug info
           
           fill(255);//fill with white box
           stroke(255);
           rect(cellsize*x,150+cellsize*y,cellsize,cellsize);
 
           fill(c);//set colour
           stroke(c);
           if(lifeField[x][y].willThisDie()){
             fill(c2);//set as dying colour
             stroke(c2);
           }
           //draw cell
           ellipse(3+cellsize*x,153+cellsize*y,cellsize-6,cellsize-6);
         }else{//blank square
           fill(255);
           stroke(255);
           print("0"); 
           rect(cellsize*x,150+cellsize*y,cellsize,cellsize);
         }
         //draw square
        
      }
      println();
    }
    println("done");
  }
  
  void initSquare(int x, int y){//turn square on x,y
    lifeField[x][y].init();
  }
  void spawnSquare(int x, int y){//gen new square x,y
    lifeField[x][y].spawn();
  }
  void killSquare(int x, int y){//deactivate square x,y
    lifeField[x][y].kill();
  }
  
  void update(){//do game of life rules
    for (int x = 0; x < gridWidth; x++){
      for (int y = 0; y < gridHeight; y++){
        int temp = checkNeighbours(x,y);//get how many active neighbour squares
        if (temp > starveThreshold || temp < communeThreshold){
          lifeField[x][y].kill();//overpopulation
        }
        if (temp == birthVal){
          lifeField[x][y].spawn();//new life
        }
      }
 //     println();
    }
    //once done checking make squares old status match present status (done to avoid checking against transformed neighbours)
    for (int x = 0; x < gridWidth; x++){
      for (int y = 0; y < gridHeight; y++){
        if (lifeField[x][y].isItAlive()){
          lifeField[x][y].spawn();
        }else{
          lifeField[x][y].kill();
        }
      }
    }
  }
  
  void checkForDeath(){
    for (int x = 0; x < gridWidth; x++){
      for (int y = 0; y < gridHeight; y++){
        int temp = checkNeighbours(x,y);//get how many active neighbour squares
        if (temp > starveThreshold || temp < communeThreshold){
          lifeField[x][y].setWillDie(true);//overpopulation
        }
        else{
          lifeField[x][y].setWillDie(false);//overpopulation
        }
      }
    }
  }
  
  int checkNeighbours(int x, int y){//how many neighbours are active
    int tempN = 0; //init at 0
    if (x > 0){
      if (y > 0){//bounds check
        if (lifeField[x-1][y-1].wasItAlive()){
          tempN++;
        }
      }
      if (lifeField[x-1][y].wasItAlive()){
         tempN++;
      }
      if (y < gridHeight-1){
        if (lifeField[x-1][y+1].wasItAlive()){
          tempN++;
        }
      }
    }
    if (y > 0){
      if (lifeField[x][y-1].wasItAlive()){
        tempN++;
      }
    }
    if (y < gridHeight-1){
      if (lifeField[x][y+1].wasItAlive()){
        tempN++;
      }
    }
    if (x < gridWidth-1){
      if (y > 0){
        if (lifeField[x+1][y-1].wasItAlive()){
          tempN++;
        }
      }
      if (lifeField[x+1][y].wasItAlive()){
         tempN++;
      }
      if (y < gridHeight-1){
        if (lifeField[x+1][y+1].wasItAlive()){
          tempN++;
        }
      }
    }
//   print(tempN);
    return tempN;//how many
    }
}

